/***************************************************************************//**
*   @file   cn0188.c
*   @brief  Implementation of CN0188 Driver for Microblaze processor.
*   @author DBogdan (dragos.bogdan@analog.com)
*
********************************************************************************
* Copyright 2012(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
********************************************************************************
*   SVN Revision: 468
*******************************************************************************/
#define CN0188_GLOBALS

/******************************************************************************/
/* Include Files                                                              */
/******************************************************************************/
#include "cn0188.h"
#include "../drivers/spi.h"
#include "../drivers/gpio.h"

/******************************************************************************/
/*  Private variables                                                         */
/******************************************************************************/
volatile unsigned int ad7171Data            = 0x0;  // Received data from AD7171.

/******************************************************************************/
/*  Aliases for the uC Probe data                                             */
/******************************************************************************/
volatile unsigned char* acquireDataButton   = &CN0188_data[0].uchData;  // Acquire Data button.
volatile unsigned int*  dataValue           = &CN0188_data[1].uiData;   // Received data from AD7171.
volatile unsigned char* correctTransfer     = &CN0188_data[2].uchData;  // Correct transfer indicator.

/***************************************************************************//**
 * @brief Reads 16 bits from SPI.
 *
 * @param base - Base address.
  * @param data - Data which will be transmitted (MOSI pin).
 *
 * @return receivedData - Received data (MISO pin).
*******************************************************************************/
unsigned int SPI_Read24Bits(unsigned int base, unsigned int data)
{
    unsigned int receivedData;

    unsigned char rxBuf[3];
    unsigned char txBuf[3] = {(data & 0x00FF0000) >> 16, (data & 0x0000FF00) >> 8, (data & 0x000000FF) >> 0};

    SPI_TransferData(CN0188_SPI, 3, (char*)txBuf, 3, (char*)rxBuf, 1);
    receivedData = (((unsigned int)rxBuf[0] << 16) + ((unsigned int)rxBuf[1] << 8) + ((unsigned int)rxBuf[2] << 0));

    return (receivedData);
}

/***************************************************************************//**
 * @brief Initializes SPI and initial values for CN0188 board.
 *
 * @param None.
 *
 * @return Always returns success.
*******************************************************************************/
int CN0188_Init(void)
{

    // Setup SPI interface.
    SPI_Init(CN0188_SPI, 0, 1, 1);

    return(TRUE);
}

/**************************************************************************//**
 * @brief Updates the device's monitored and controlled parameters.
 *
 * @param None.
 *
 * @return Always returns success.
******************************************************************************/
int CN0188_UpdateParameters(void)
{
    if(*acquireDataButton)  // If the Acquire Data button was pressed,
    {                       // reads data from AD7171.
        ad7171Data = SPI_Read24Bits(CN0188_SPI, 0x000000);
        *dataValue = (ad7171Data & 0xFFFF00) >> 8;
        if((ad7171Data & 0x7) == AD7171_DEFAULT_PATTERN)
        {
            *correctTransfer = 1;
        }
        else
        {
            *correctTransfer = 0;
        }
        *acquireDataButton = 0;
    }

    return (TRUE);
}
